#pragma once
#include "AV_UnrealExtraTypes.h"
#ifdef IS_UNREAL
#include "../UT/UT_SerializationContext.h"
#include "../UT/UT_SerializationData.h"
#include "Engine/Light.h"
#include "Engine/StaticMeshActor.h"
#include "UE_stdafx.h"
#else
#include "AV_ExtraTypes.h"
#include "AV_UnrealActorLight.h"
#include "AV_UnrealActorStaticMesh.h"
#endif

//DECLARE_DELEGATE_RetVal(class UPXConnectMain*, FGetMainDelegate);
//FGetMainDelegate& getGetMainDelegate();

#ifdef IS_UNREAL
class UPXConnectRootObjects;
#endif // IS_UNREAL


namespace AV
{

#ifdef IS_UNREAL
	
#else
class UnrealWorld;
class UnrealActor;
#endif

namespace UnrealSerializationHelper
{
class UeSerStructFactory;

struct UnrealSerStruct
{
	UnrealSerStruct();
	virtual ~UnrealSerStruct();
	virtual void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr){return;};
};

struct UnrealVec3 : UnrealSerStruct
{
	UnrealVec3();
	UnrealVec3(float inX,float inY,float inZ);
#ifndef IS_UNREAL
	UnrealVec3(const mlVec3& inVec);
#else
	UnrealVec3(const FVector& inVec);
#endif
	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;
#ifndef IS_UNREAL
	virtual mlVec3 asMlVec() const;
#else
	FVector asFvector() const;
#endif
	float x;
	float y;
	float z;
};

struct UnrealVec3Scale : UnrealVec3
{
	UnrealVec3Scale();
#ifndef IS_UNREAL
	using UnrealVec3::UnrealVec3;
#else
	using UnrealVec3::UnrealVec3;
#endif
	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;
};

//Struct for helping transferring Positional Data from Pixera to UE an back.
//Data will be converted to match the present coordinate System
struct UnrealVec3Pos : UnrealVec3
{
	UnrealVec3Pos();
#ifndef IS_UNREAL
	using UnrealVec3::UnrealVec3;
#else
	using UnrealVec3::UnrealVec3;
#endif
	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;
};

//Struct for helping transferring Rotation Data from Pixera to UE an back.
//Data will be converted to match the present coordinate System
struct UnrealVec3Rot : UnrealVec3
{
	UnrealVec3Rot();
#ifndef IS_UNREAL
	using UnrealVec3::UnrealVec3;
#else
	using UnrealVec3::UnrealVec3;
	UnrealVec3Rot(const FRotator& inRotator);
#endif
#ifndef IS_UNREAL
	mlVec3 asMlVec() const override;
#else
	FRotator asFRotator() const;
#endif
	void serialize(UT::SerializationContext& ctx, UeSerStructFactory* pFactory = nullptr) override;
};

struct UnrealActorSerializationStruct : UnrealSerStruct
{
	UnrealActorSerializationStruct();
#ifndef IS_UNREAL
	UnrealActorSerializationStruct(UnrealActor* pActor);
#else
	UnrealActorSerializationStruct(AActor* pActor);
#endif
	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;

	std::string objectName;
	UnrealVec3Pos position;
	UnrealVec3Rot rotation;
	UnrealVec3Scale scale;
	unsigned long long parentWorldInterthreadIdDiff;
	UnrealMobilityTypes eMobilityType;
	bool isVisible;
};

struct UnrealActorLightSerializationStruct : UnrealActorSerializationStruct
{
	UnrealActorLightSerializationStruct();
#ifndef IS_UNREAL
	UnrealActorLightSerializationStruct(UnrealActorLight* pActor);
#else
	UnrealActorLightSerializationStruct(ALight* pLight);
#endif // !IS_UNREAL

	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;

	UnrealLightTypes eLightType;
	float brightness;
	UnrealVec3 lightColor;
	float colorTemperature;
	bool useColorTemperature;
	//Directional Light
	int32 atmosphereSunLightIndex;
	//Local Light Comp
	float attenuationRadius;
	UnrealLightUnits eUnit;
	//Point Light
	bool useInverseSquaredFalloff;
	float lightFalloffExponent;
	float softSourceRadius;
	float sourceLength;
	float sourceRadius;
	//Spot Light
	float innerConeAngle;
	float outerConeAngle;
	//Rect Light
	float barnDoorAngle;
	float barnDoorLength;
	float sourceHeight;
	float sourceWidth;

};

struct UnrealActorStaticMeshSerializationStruct : UnrealActorSerializationStruct
{
	UnrealActorStaticMeshSerializationStruct();
#ifndef IS_UNREAL
	UnrealActorStaticMeshSerializationStruct(UnrealActorStaticMesh* pActor);
#else
	UnrealActorStaticMeshSerializationStruct(AStaticMeshActor* pStaticMesh);
#endif

	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;
};

struct UnrealWorldSerializationStruct : UnrealSerStruct
{
	UnrealWorldSerializationStruct();
#ifndef IS_UNREAL
	UnrealWorldSerializationStruct(UnrealWorld* pWorld);
#else
	UnrealWorldSerializationStruct(UPXConnectRootObjects* pRootObjects);
#endif
	~UnrealWorldSerializationStruct();
	void serialize(UT::SerializationContext& ctx,UeSerStructFactory* pFactory=nullptr) override;
	std::string worldName;
	unsigned long long worldInterthreadIdDiff;
	std::unique_ptr<std::vector<UnrealActorSerializationStruct*>> pChildActors;
};

template <class Base>
class UeSerStructInstantiatorBase
{
public:
	virtual Base* create()
	{
		return nullptr;
	}
};

template <class Base,class Derived>
class UeSerStructInstantiator : public UeSerStructInstantiatorBase<Base>
{
public:
	Base* create() override
	{
		return new Derived();
	}
};

class UeInstNameComp
{
public:
	bool operator()(const char* pA, const char* pB) const { return strcmp(pA, pB) < 0; };
};

template <class Base>
using UeStructInstatiatorMap = std::map <const char*, UeSerStructInstantiatorBase<Base>*, UeInstNameComp>;

typedef UeSerStructInstantiatorBase<UnrealSerStruct> UeSerStructInst;
typedef UeStructInstatiatorMap<UnrealSerStruct> UeSerStructInstMap;
class UeSerStructFactory
{
public:
	UeSerStructFactory();
	~UeSerStructFactory();
	static UeSerStructFactory& instance();
	UnrealSerStruct* create(const char* pName);
protected:
	void addInstatiator(const char* pName,UeSerStructInst* pInst);
	UeSerStructInstMap m_instMap;
};

void deserializeNetworkPacket(UT::SerializationContext& ctx);
#ifdef IS_UNREAL
void serializeAndSend(UnrealSerStruct* pSerStruct);
UnrealActorSerializationStruct* getSerStructForActor(AActor* pActor);
#endif
#ifndef IS_UNREAL
UnrealActor* createActorFromSerStruct(UnrealActorSerializationStruct* pSerStruct);
#endif
}



}

